 
#include "stdafx.h"  
#include <math.h>  
#include <vector>  
#include <iostream>  
#include "tbb/task_scheduler_init.h"  
#include "tbb/parallel_sort.h"  
#include "tbb/tick_count.h"
  
using namespace tbb;  
using namespace std;  
  

FILE* fout;
class point {  
public:  
    double val;   
	//   
    bool operator< (const point& that) const { return val < that.val; }  
};  

int test (int size)
{
	//       
	vector<point> pointsSeq(size);  
	vector<point> pointsPar(size);  
    for(size_t i=0; i<pointsSeq.size(); i++) {  
        pointsSeq[i].val = rand();
		pointsPar[i].val = pointsSeq[i].val;
    }  
	
	//     
	tick_count serial_start = tick_count::now();  
    std::sort(pointsSeq.begin(), pointsSeq.end(), std::less<point>());  
    tick_count serial_end = tick_count::now();  
    
	// tbb 
	task_scheduler_init init;  
    
	//    
	tick_count parallel_start = tick_count::now();
	// 
    parallel_sort(pointsPar.begin(), pointsPar.end(), std::less<point>());  
	//    
	tick_count parallel_end = tick_count::now();  
    
	//     
	for(size_t i=1; i<pointsPar.size(); i++)  
        if(pointsSeq[i].val < pointsSeq[i-1].val) {  
            cout << endl << "Error: Serial sort is not correct starting at location "<< i;  
            fprintf (fout ,"\n\nError: Serial sort is not correct starting at location %d \n",i);  
			return 1;  
        }
	//     
	for(size_t i=1; i<pointsPar.size(); i++)  
        if(pointsPar[i].val < pointsPar[i-1].val) {  
            cout << endl << "Error: Parallel sort is not correct starting at location " << i;  
            fprintf (fout ,"\n\nError: Parallel sort is not correct starting at location \n",i);  
			return 2;  
        }
  
	//  
    //for(size_t i=0; i<points.size(); i++)  
    //    cout << "Seq " << pointsSeq[i].val << "  Par " << pointsPar[i].val << endl;  
	
	//  :  ,          .
	fprintf (fout, "\nSize = %d \nSequential time = %f \nParallel time = %f \nSpeedup of parallelism is %f", size, (serial_end - serial_start).seconds(), (parallel_end - parallel_start).seconds(),  (serial_end - serial_start).seconds() / (parallel_end - parallel_start).seconds());
	cout << endl << "Size = " << size 
		<< endl << "Sequential time = " << (serial_end - serial_start).seconds()
		<< endl << "Parallel time = " << (parallel_end - parallel_start).seconds()
		<< endl << "Speedup of parallelism is " << ((serial_end - serial_start).seconds() / (parallel_end - parallel_start).seconds())
		<< endl;
}
int main(int argc, char* argv[])  
{	int res;
	fout = fopen ("test log.txt", "a");
	
	//   
	for (int i=10000; i<10000000; i+=i/2)
	{	
		res = test(i);
	}
	cout << "Tests finished, you can also see information above in 'tests log.txt'";
	getchar();
    return res;  
}  